@@ -3,23 +3,27 @@
 import argparse
 import io
 import json
-import gzip
 import logging
 import os
 import sys
 import copy
 
-from datetime import datetime
-from decimal import Decimal
-from tempfile import mkstemp
-from typing import Dict
-from dateutil import parser
-from dateutil.parser import ParserError
+from typing import Dict, List
 from joblib import Parallel, delayed, parallel_backend
 from jsonschema import Draft7Validator, FormatChecker
 from singer import get_logger
 
+import target_snowflake.file_formats.csv as csv
+import target_snowflake.file_formats.parquet as parquet
+import target_snowflake.stream_utils as stream_utils
+
 from target_snowflake.db_sync import DbSync
+from target_snowflake.file_format import FileFormatTypes
+from target_snowflake.exceptions import (
+    RecordValidationException,
+    UnexpectedValueTypeException,
+    InvalidValidationOperationException
+)
 
 LOGGER = get_logger('target_snowflake')
 
@@ -30,33 +34,6 @@ DEFAULT_BATCH_SIZE_ROWS = 100000
 DEFAULT_PARALLELISM = 0  # 0 The number of threads used to flush tables
 DEFAULT_MAX_PARALLELISM = 16  # Don't use more than this number of threads by default when flushing streams in parallel
 
-# max timestamp/datetime supported in SF, used to reset all invalid dates that are beyond this value
-MAX_TIMESTAMP = '9999-12-31 23:59:59.999999'
-
-# max time supported in SF, used to reset all invalid times that are beyond this value
-MAX_TIME = '23:59:59.999999'
-
-
-class RecordValidationException(Exception):
-    """Exception to raise when record validation failed"""
-    pass
-
-
-class InvalidValidationOperationException(Exception):
-    """Exception to raise when internal JSON schema validation process failed"""
-    pass
-
-
-def float_to_decimal(value):
-    """Walk the given data structure and turn all instances of float into double."""
-    if isinstance(value, float):
-        return Decimal(str(value))
-    if isinstance(value, list):
-        return [float_to_decimal(child) for child in value]
-    if isinstance(value, dict):
-        return {k: float_to_decimal(v) for k, v in value.items()}
-    return value
-
 
 def add_metadata_columns_to_schema(schema_message):
     """Metadata _sdc columns according to the stitch documentation at
@@ -74,84 +51,31 @@ def add_metadata_columns_to_schema(schema_message):
     return extended_schema_message
 
 
-def add_metadata_values_to_record(record_message, stream_to_sync):
-    """Populate metadata _sdc columns from incoming record message
-    The location of the required attributes are fixed in the stream
-    """
-    extended_record = record_message['record']
-    extended_record['_sdc_extracted_at'] = record_message.get('time_extracted')
-    extended_record['_sdc_batched_at'] = datetime.now().isoformat()
-    extended_record['_sdc_deleted_at'] = record_message.get('record', {}).get('_sdc_deleted_at')
-
-    return extended_record
-
-
 def emit_state(state):
+    """Print state to stdout"""
     if state is not None:
         line = json.dumps(state)
-        LOGGER.info('Emitting state {}'.format(line))
+        LOGGER.info('Emitting state %s', line)
         sys.stdout.write("{}\n".format(line))
         sys.stdout.flush()
 
 
-def get_schema_names_from_config(config):
-    default_target_schema = config.get('default_target_schema')
-    schema_mapping = config.get('schema_mapping', {})
-    schema_names = []
-
-    if default_target_schema:
-        schema_names.append(default_target_schema)
-
-    if schema_mapping:
-        for source_schema, target in schema_mapping.items():
-            schema_names.append(target.get('target_schema'))
-
-    return schema_names
-
-
 def load_table_cache(config):
+    """Load table cache from snowflake metadata"""
     table_cache = []
     if not ('disable_table_cache' in config and config['disable_table_cache']):
-        LOGGER.info("Getting catalog objects from table cache...")
+        LOGGER.info('Getting catalog objects from table cache...')
 
-        db = DbSync(config)
+        db = DbSync(config) # pylint: disable=invalid-name
         table_cache = db.get_table_columns(
-            table_schemas=get_schema_names_from_config(config))
+            table_schemas=stream_utils.get_schema_names_from_config(config))
 
     return table_cache
 
 
-def adjust_timestamps_in_record(record: Dict, schema: Dict) -> None:
-    """
-    Goes through every field that is of type date/datetime/time and if its value is out of range,
-    resets it to MAX value accordingly
-    Args:
-        record: record containing properties and values
-        schema: json schema that has types of each property
-    """
-
-    # creating this internal function to avoid duplicating code and too many nested blocks.
-    def reset_new_value(record: Dict, key: str, format: str):
-        try:
-            parser.parse(record[key])
-        except ParserError:
-            record[key] = MAX_TIMESTAMP if format != 'time' else MAX_TIME
-
-    for key, value in record.items():
-        if value is not None and key in schema['properties']:
-            if 'anyOf' in schema['properties'][key]:
-                for type_dict in schema['properties'][key]['anyOf']:
-                    if 'string' in type_dict['type'] and type_dict.get('format', None) in {'date-time', 'time', 'date'}:
-                        reset_new_value(record, key, type_dict['format'])
-                        break
-            else:
-                if 'string' in schema['properties'][key]['type'] and \
-                        schema['properties'][key].get('format', None) in {'date-time', 'time', 'date'}:
-                    reset_new_value(record, key, schema['properties'][key]['format'])
-
-
-# pylint: disable=too-many-locals,too-many-branches,too-many-statements
+# pylint: disable=too-many-locals,too-many-branches,too-many-statements,invalid-name
 def persist_lines(config, lines, table_cache=None) -> None:
+    """Main loop to read and consume singer messages from stdin"""
     state = None
     flushed_state = None
     schemas = {}
@@ -168,37 +92,38 @@ def persist_lines(config, lines, table_cache=None) -> None:
         try:
             o = json.loads(line)
         except json.decoder.JSONDecodeError:
-            LOGGER.error("Unable to parse:\n{}".format(line))
+            LOGGER.error('Unable to parse:\n%s', line)
             raise
 
         if 'type' not in o:
-            raise Exception("Line is missing required key 'type': {}".format(line))
+            raise Exception(f"Line is missing required key 'type': {line}")
 
         t = o['type']
 
         if t == 'RECORD':
             if 'stream' not in o:
-                raise Exception("Line is missing required key 'stream': {}".format(line))
+                raise Exception(f"Line is missing required key 'stream': {line}")
             if o['stream'] not in schemas:
                 raise Exception(
-                    "A record for stream {} was encountered before a corresponding schema".format(o['stream']))
+                    f"A record for stream {o['stream']} was encountered before a corresponding schema")
 
             # Get schema for this record's stream
             stream = o['stream']
 
-            adjust_timestamps_in_record(o['record'], schemas[stream])
+            stream_utils.adjust_timestamps_in_record(o['record'], schemas[stream])
 
             # Validate record
             if config.get('validate_records'):
                 try:
-                    validators[stream].validate(float_to_decimal(o['record']))
+                    validators[stream].validate(stream_utils.float_to_decimal(o['record']))
                 except Exception as ex:
                     if type(ex).__name__ == "InvalidOperation":
                         raise InvalidValidationOperationException(
                             f"Data validation failed and cannot load to destination. RECORD: {o['record']}\n"
                             "multipleOf validations that allows long precisions are not supported (i.e. with 15 digits"
-                            "or more) Try removing 'multipleOf' methods from JSON schema.")
-                    raise RecordValidationException(f"Record does not pass schema validation. RECORD: {o['record']}")
+                            "or more) Try removing 'multipleOf' methods from JSON schema.") from ex
+                    raise RecordValidationException(f"Record does not pass schema validation. RECORD: {o['record']}") \
+                        from ex
 
             primary_key_string = stream_to_sync[stream].record_primary_key_string(o['record'])
             if not primary_key_string:
@@ -214,7 +139,7 @@ def persist_lines(config, lines, table_cache=None) -> None:
 
             # append record
             if config.get('add_metadata_columns') or config.get('hard_delete'):
-                records_to_load[stream][primary_key_string] = add_metadata_values_to_record(o, stream_to_sync[stream])
+                records_to_load[stream][primary_key_string] = stream_utils.add_metadata_values_to_record(o)
             else:
                 records_to_load[stream][primary_key_string] = o['record']
 
@@ -240,10 +165,10 @@ def persist_lines(config, lines, table_cache=None) -> None:
 
         elif t == 'SCHEMA':
             if 'stream' not in o:
-                raise Exception("Line is missing required key 'stream': {}".format(line))
+                raise Exception(f"Line is missing required key 'stream': {line}")
 
             stream = o['stream']
-            new_schema = float_to_decimal(o['schema'])
+            new_schema = stream_utils.float_to_decimal(o['schema'])
 
             # Update and flush only if the the schema is new or different than
             # the previously used version of the schema
@@ -256,7 +181,12 @@ def persist_lines(config, lines, table_cache=None) -> None:
                 # if same stream has been encountered again, it means the schema might have been altered
                 # so previous records need to be flushed
                 if row_count.get(stream, 0) > 0:
-                    flushed_state = flush_streams(records_to_load, row_count, stream_to_sync, config, state, flushed_state)
+                    flushed_state = flush_streams(records_to_load,
+                                                  row_count,
+                                                  stream_to_sync,
+                                                  config,
+                                                  state,
+                                                  flushed_state)
 
                     # emit latest encountered state
                     emit_state(flushed_state)
@@ -274,7 +204,7 @@ def persist_lines(config, lines, table_cache=None) -> None:
                 #  or
                 #  2) Use fastsync [postgres-to-snowflake, mysql-to-snowflake, etc.]
                 if config.get('primary_key_required', True) and len(o['key_properties']) == 0:
-                    LOGGER.critical("Primary key is set to mandatory but not defined in the [{}] stream".format(stream))
+                    LOGGER.critical('Primary key is set to mandatory but not defined in the [%s] stream', stream)
                     raise Exception("key_properties field is required")
 
                 key_properties[stream] = o['key_properties']
@@ -294,7 +224,7 @@ def persist_lines(config, lines, table_cache=None) -> None:
             LOGGER.debug('ACTIVATE_VERSION message')
 
         elif t == 'STATE':
-            LOGGER.debug('Setting state to {}'.format(o['value']))
+            LOGGER.debug('Setting state to %s', o['value'])
             state = o['value']
 
             # Initially set flushed state
@@ -302,8 +232,7 @@ def persist_lines(config, lines, table_cache=None) -> None:
                 flushed_state = copy.deepcopy(state)
 
         else:
-            raise Exception("Unknown message type {} in message {}"
-                            .format(o['type'], o))
+            raise Exception(f"Unknown message type {o['type']} in message {o}")
 
     # if some bucket has records that need to be flushed but haven't reached batch size
     # then flush all buckets.
@@ -360,7 +289,7 @@ def flush_streams(
     with parallel_backend('threading', n_jobs=parallelism):
         Parallel()(delayed(load_stream_batch)(
             stream=stream,
-            records_to_load=streams[stream],
+            records=streams[stream],
             row_count=row_count,
             db_sync=stream_to_sync[stream],
             no_compression=config.get('no_compression'),
@@ -390,11 +319,12 @@ def flush_streams(
     return flushed_state
 
 
-def load_stream_batch(stream, records_to_load, row_count, db_sync, no_compression=False, delete_rows=False,
+def load_stream_batch(stream, records, row_count, db_sync, no_compression=False, delete_rows=False,
                       temp_dir=None):
+    """Load one batch of the stream into target table"""
     # Load into snowflake
     if row_count[stream] > 0:
-        flush_records(stream, records_to_load, row_count[stream], db_sync, temp_dir, no_compression)
+        flush_records(stream, records, db_sync, temp_dir, no_compression)
 
         # Delete soft-deleted, flagged rows - where _sdc_deleted at is not null
         if delete_rows:
@@ -404,36 +334,49 @@ def load_stream_batch(stream, records_to_load, row_count, db_sync, no_compressio
         row_count[stream] = 0
 
 
-def write_record_to_file(outfile, records_to_load, record_to_csv_line_transformer):
-    for record in records_to_load.values():
-        csv_line = record_to_csv_line_transformer(record)
-        outfile.write(bytes(csv_line + '\n', 'UTF-8'))
+def flush_records(stream: str,
+                  records: List[Dict],
+                  db_sync: DbSync,
+                  temp_dir: str = None,
+                  no_compression: bool = False) -> None:
+    """
+    Takes a list of record messages and loads it into the snowflake target table
 
+    Args:
+        stream: Name of the stream
+        records: List of dictionary, that represents multiple csv lines. Dict key is the column name, value is the
+                 column value
+        row_count:
+        db_sync: A DbSync object
+        temp_dir: Directory where intermediate temporary files will be created. (Default: OS specificy temp directory)
+        no_compression: Disable to use compressed files. (Default: False)
 
-def flush_records(stream, records_to_load, row_count, db_sync, temp_dir=None, no_compression=False):
-    if temp_dir:
-        os.makedirs(temp_dir, exist_ok=True)
-    csv_fd, csv_file = mkstemp(suffix='.csv', prefix='records_', dir=temp_dir)
-    record_to_csv_line_transformer = db_sync.record_to_csv_line
+    Returns:
+        None
+    """
+    # Generate file on disk in the required format
+    filepath = db_sync.file_format.formatter.records_to_file(records,
+                                                             db_sync.flatten_schema,
+                                                             compression=not no_compression,
+                                                             dest_dir=temp_dir,
+                                                             data_flattening_max_level=
+                                                                db_sync.data_flattening_max_level)
 
-    # Using gzip or plain file object
-    if no_compression:
-        with open(csv_fd, 'wb') as outfile:
-            write_record_to_file(outfile, records_to_load, record_to_csv_line_transformer)
-    else:
-        with open(csv_fd, 'wb') as outfile:
-            with gzip.GzipFile(filename=csv_file, mode='wb',fileobj=outfile) as gzipfile:
-                write_record_to_file(gzipfile, records_to_load, record_to_csv_line_transformer)
+    # Get file stats
+    row_count = len(records)
+    size_bytes = os.path.getsize(filepath)
 
-    size_bytes = os.path.getsize(csv_file)
-    s3_key = db_sync.put_to_stage(csv_file, stream, row_count, temp_dir=temp_dir)
-    db_sync.load_csv(s3_key, row_count, size_bytes)
+    # Upload to s3 and load into Snowflake
+    s3_key = db_sync.put_to_stage(filepath, stream, row_count, temp_dir=temp_dir)
+    db_sync.load_file(s3_key, row_count, size_bytes)
 
-    os.remove(csv_file)
+    # Delete file from local disk and from s3
+    os.remove(filepath)
     db_sync.delete_from_stage(stream, s3_key)
 
 
 def main():
+    """Main function"""
     arg_parser = argparse.ArgumentParser()
     arg_parser.add_argument('-c', '--config', help='Config file')
     args = arg_parser.parse_args()
