@@ -1,26 +1,23 @@
 import json
 import sys
-from typing import Tuple, List, Dict, Optional, Union
+from typing import List, Dict, Union
 
 import snowflake.connector
-import collections
-import inflection
 import re
-import itertools
 import time
 
 from singer import get_logger
+import target_snowflake.flattening as flattening
+import target_snowflake.stream_utils as stream_utils
+from target_snowflake.file_format import FileFormat, FileFormatTypes
 
-from target_snowflake.s3_upload_client import S3UploadClient
-from target_snowflake.snowflake_upload_client import SnowflakeUploadClient
-
-
-class TooManyRecordsException(Exception):
-    """Exception to raise when query returns more records than max_records"""
-    pass
+from target_snowflake.exceptions import TooManyRecordsException
+from target_snowflake.upload_clients.s3_upload_client import S3UploadClient
+from target_snowflake.upload_clients.snowflake_upload_client import SnowflakeUploadClient
 
 
 def validate_config(config):
+    """Validate configuration"""
     errors = []
     s3_required_config_keys = [
         'account',
@@ -66,158 +63,67 @@ def validate_config(config):
     if not config_default_target_schema and not config_schema_mapping:
         errors.append("Neither 'default_target_schema' (string) nor 'schema_mapping' (object) keys set in config.")
 
-    # Check client-side encryption config
-    config_cse_key = config.get('client_side_encryption_master_key', None)
-
     return errors
 
 
 def column_type(schema_property):
+    """Take a specific schema property and return the snowflake equivalent column type"""
     property_type = schema_property['type']
     property_format = schema_property['format'] if 'format' in schema_property else None
-    column_type = 'text'
+    col_type = 'text'
     if 'object' in property_type or 'array' in property_type:
-        column_type = 'variant'
+        col_type = 'variant'
 
     # Every date-time JSON value is currently mapped to TIMESTAMP_NTZ
-    #
-    # TODO: Detect if timezone postfix exists in the JSON and find if TIMESTAMP_TZ or
-    # TIMSTAMP_NTZ is the better column type
     elif property_format == 'date-time':
-        column_type = 'timestamp_ntz'
+        col_type = 'timestamp_ntz'
     elif property_format == 'time':
-        column_type = 'time'
+        col_type = 'time'
     elif property_format == 'binary':
-        column_type = 'binary'
+        col_type = 'binary'
     elif 'number' in property_type:
-        column_type = 'float'
+        col_type = 'float'
     elif 'integer' in property_type and 'string' in property_type:
-        column_type = 'text'
+        col_type = 'text'
     elif 'integer' in property_type:
-        column_type = 'number'
+        col_type = 'number'
     elif 'boolean' in property_type:
-        column_type = 'boolean'
+        col_type = 'boolean'
 
-    return column_type
+    return col_type
 
 
 def column_trans(schema_property):
+    """Generate SQL transformed columns syntax"""
     property_type = schema_property['type']
-    column_trans = ''
+    col_trans = ''
     if 'object' in property_type or 'array' in property_type:
-        column_trans = 'parse_json'
+        col_trans = 'parse_json'
     elif schema_property.get('format') == 'binary':
-        column_trans = 'to_binary'
+        col_trans = 'to_binary'
 
-    return column_trans
+    return col_trans
 
 
 def safe_column_name(name):
+    """Generate SQL friendly column name"""
     return '"{}"'.format(name).upper()
 
+def json_element_name(name):
+    """Generate SQL friendly semi structured element reference name"""
+    return '"{}"'.format(name)
 
 def column_clause(name, schema_property):
+    """Generate DDL column name with column type string"""
     return '{} {}'.format(safe_column_name(name), column_type(schema_property))
 
 
-def flatten_key(k, parent_key, sep):
-    full_key = parent_key + [k]
-    inflected_key = full_key.copy()
-    reducer_index = 0
-    while len(sep.join(inflected_key)) >= 255 and reducer_index < len(inflected_key):
-        reduced_key = re.sub(r'[a-z]', '', inflection.camelize(inflected_key[reducer_index]))
-        inflected_key[reducer_index] = \
-            (reduced_key if len(reduced_key) > 1 else inflected_key[reducer_index][0:3]).lower()
-        reducer_index += 1
-
-    return sep.join(inflected_key)
-
-
-def flatten_schema(d, parent_key=[], sep='__', level=0, max_level=0):
-    items = []
-
-    if 'properties' not in d:
-        return {}
-
-    for k, v in d['properties'].items():
-        new_key = flatten_key(k, parent_key, sep)
-        if 'type' in v.keys():
-            if 'object' in v['type'] and 'properties' in v and level < max_level:
-                items.extend(flatten_schema(v, parent_key + [k], sep=sep, level=level + 1, max_level=max_level).items())
-            else:
-                items.append((new_key, v))
-        else:
-            if len(v.values()) > 0:
-                if list(v.values())[0][0]['type'] == 'string':
-                    list(v.values())[0][0]['type'] = ['null', 'string']
-                    items.append((new_key, list(v.values())[0][0]))
-                elif list(v.values())[0][0]['type'] == 'array':
-                    list(v.values())[0][0]['type'] = ['null', 'array']
-                    items.append((new_key, list(v.values())[0][0]))
-                elif list(v.values())[0][0]['type'] == 'object':
-                    list(v.values())[0][0]['type'] = ['null', 'object']
-                    items.append((new_key, list(v.values())[0][0]))
-
-    key_func = lambda item: item[0]
-    sorted_items = sorted(items, key=key_func)
-    for k, g in itertools.groupby(sorted_items, key=key_func):
-        if len(list(g)) > 1:
-            raise ValueError('Duplicate column name produced in schema: {}'.format(k))
-
-    return dict(sorted_items)
-
-
-def _should_json_dump_value(key, value, flatten_schema=None):
-    if isinstance(value, (dict, list)):
-        return True
-
-    if flatten_schema and key in flatten_schema and 'type' in flatten_schema[key] and set(
-            flatten_schema[key]['type']) == {'null', 'object', 'array'}:
-        return True
-
-    return False
-
-
-# pylint: disable-msg=too-many-arguments
-def flatten_record(d, flatten_schema=None, parent_key=[], sep='__', level=0, max_level=0):
-    items = []
-    for k, v in d.items():
-        new_key = flatten_key(k, parent_key, sep)
-        if isinstance(v, collections.MutableMapping) and level < max_level:
-            items.extend(flatten_record(v, flatten_schema, parent_key + [k], sep=sep, level=level + 1,
-                                        max_level=max_level).items())
-        else:
-            items.append((new_key, json.dumps(v) if _should_json_dump_value(k, v, flatten_schema) else v))
-
-    return dict(items)
-
-
 def primary_column_names(stream_schema_message):
+    """Generate list of SQL friendly PK column names"""
     return [safe_column_name(p) for p in stream_schema_message['key_properties']]
 
 
-def stream_name_to_dict(stream_name, separator='-'):
-    catalog_name = None
-    schema_name = None
-    table_name = stream_name
-
-    # Schema and table name can be derived from stream if it's in <schema_nama>-<table_name> format
-    s = stream_name.split(separator)
-    if len(s) == 2:
-        schema_name = s[0]
-        table_name = s[1]
-    if len(s) > 2:
-        catalog_name = s[0]
-        schema_name = s[1]
-        table_name = '_'.join(s[2:])
-
-    return {
-        'catalog_name': catalog_name,
-        'schema_name': schema_name,
-        'table_name': table_name
-    }
-
-
+# pylint: disable=invalid-name
 def create_query_tag(query_tag_pattern: str, database: str = None, schema: str = None, table: str = None) -> str:
     """
     Generate a string to tag executed queries in Snowflake.
@@ -254,6 +160,8 @@ def create_query_tag(query_tag_pattern: str, database: str = None, schema: str =
 
 # pylint: disable=too-many-public-methods,too-many-instance-attributes
 class DbSync:
+    """DbSync class"""
+
     def __init__(self, connection_config, stream_schema_message=None, table_cache=None):
         """
             connection_config:      Snowflake connection details
@@ -285,11 +193,11 @@ class DbSync:
 
         # Exit if config has errors
         if len(config_errors) > 0:
-            self.logger.error("Invalid configuration:\n   * {}".format('\n   * '.join(config_errors)))
+            self.logger.error('Invalid configuration:\n   * %s', '\n   * '.join(config_errors))
             sys.exit(1)
 
         if self.connection_config.get('stage', None):
-            stage = stream_name_to_dict(self.connection_config['stage'], separator='.')
+            stage = stream_utils.stream_name_to_dict(self.connection_config['stage'], separator='.')
             if not stage['schema_name']:
                 self.logger.error(
                     "The named external stage object in config has to use the <schema>.<stage_name> format.")
@@ -297,8 +205,14 @@ class DbSync:
 
         self.schema_name = None
         self.grantees = None
+        self.file_format = FileFormat(self.connection_config['file_format'], self.query)
 
-        # Init stream schema
+        if not self.connection_config.get('stage') and self.file_format.file_format_type == FileFormatTypes.PARQUET:
+            self.logger.error("Table stages with Parquet file format is not suppported. "
+                              "Use named stages with Parquet file format or table stages with CSV files format")
+            sys.exit(1)
+
+        # Init stream schema pylint: disable=line-too-long
         if self.stream_schema_message is not None:
             #  Define target schema name.
             #  --------------------------
@@ -319,7 +233,7 @@ class DbSync:
             config_schema_mapping = self.connection_config.get('schema_mapping', {})
 
             stream_name = stream_schema_message['stream']
-            stream_schema_name = stream_name_to_dict(stream_name)['schema_name']
+            stream_schema_name = stream_utils.stream_name_to_dict(stream_name)['schema_name']
             if config_schema_mapping and stream_schema_name in config_schema_mapping:
                 self.schema_name = config_schema_mapping[stream_schema_name].get('target_schema')
             elif config_default_target_schema:
@@ -353,12 +267,12 @@ class DbSync:
                                                                               self.grantees)
 
             self.data_flattening_max_level = self.connection_config.get('data_flattening_max_level', 0)
-            self.flatten_schema = flatten_schema(stream_schema_message['schema'],
-                                                 max_level=self.data_flattening_max_level)
+            self.flatten_schema = flattening.flatten_schema(stream_schema_message['schema'],
+                                                            max_level=self.data_flattening_max_level)
 
         # Use external stage
         if connection_config.get('s3_bucket', None):
-            self.uploadClient = S3UploadClient(connection_config)
+            self.upload_client = S3UploadClient(connection_config)
         # Use table stage
         else:
             # Enforce no parallelism with table stages.
@@ -372,9 +286,10 @@ class DbSync:
                                     'key to the configuration')
                 connection_config['parallelism'] = 1
 
-            self.uploadClient = SnowflakeUploadClient(connection_config, self)
+            self.upload_client = SnowflakeUploadClient(connection_config, self)
 
     def open_connection(self):
+        """Open snowflake connection"""
         stream = None
         if self.stream_schema_message:
             stream = self.stream_schema_message['stream']
@@ -398,6 +313,7 @@ class DbSync:
         )
 
     def query(self, query: Union[str, List[str]], params: Dict = None, max_records=0) -> List[Dict]:
+        """Run an SQL query in snowflake"""
         result = []
 
         if params is None:
@@ -411,7 +327,7 @@ class DbSync:
             with connection.cursor(snowflake.connector.DictCursor) as cur:
 
                 # Run every query in one transaction if query is a list of SQL
-                if type(query) is list:
+                if isinstance(query, list):
                     self.logger.info('Starting Transaction')
                     cur.execute("START TRANSACTION")
                     queries = query
@@ -420,6 +336,7 @@ class DbSync:
 
                 qid = None
 
+                # pylint: disable=invalid-name
                 for q in queries:
 
                     # update the LAST_QID
@@ -440,10 +357,11 @@ class DbSync:
         return result
 
     def table_name(self, stream_name, is_temporary, without_schema=False):
+        """Generate target table name"""
         if not stream_name:
             return None
 
-        stream_dict = stream_name_to_dict(stream_name)
+        stream_dict = stream_utils.stream_name_to_dict(stream_name)
         table_name = stream_dict['table_name']
         sf_table_name = table_name.replace('.', '_').replace('-', '_').lower()
 
@@ -456,38 +374,31 @@ class DbSync:
         return f'{self.schema_name}."{sf_table_name.upper()}"'
 
     def record_primary_key_string(self, record):
+        """Generate a unique PK string in the record"""
         if len(self.stream_schema_message['key_properties']) == 0:
             return None
-        flatten = flatten_record(record, self.flatten_schema, max_level=self.data_flattening_max_level)
+        flatten = flattening.flatten_record(record, self.flatten_schema, max_level=self.data_flattening_max_level)
         try:
             key_props = [str(flatten[p]) for p in self.stream_schema_message['key_properties']]
         except Exception as exc:
             self.logger.error(
-                "Cannot find {} primary key(s) in record: {}".format(self.stream_schema_message['key_properties'],
-                                                                     flatten))
+                'Cannot find %s primary key(s) in record: %s', self.stream_schema_message['key_properties'],
+                flatten)
             raise exc
         return ','.join(key_props)
 
-    def record_to_csv_line(self, record):
-        flatten = flatten_record(record, self.flatten_schema, max_level=self.data_flattening_max_level)
-
-        return ','.join(
-            [
-                json.dumps(flatten[name], ensure_ascii=False) if name in flatten and (
-                            flatten[name] == 0 or flatten[name]) else ''
-                for name in self.flatten_schema
-            ]
-        )
-
     def put_to_stage(self, file, stream, count, temp_dir=None):
-        self.logger.info("Uploading {} rows to stage".format(count))
-        return self.uploadClient.upload_file(file, stream, temp_dir)
+        """Upload file to snowflake stage"""
+        self.logger.info('Uploading %d rows to stage', count)
+        return self.upload_client.upload_file(file, stream, temp_dir)
 
     def delete_from_stage(self, stream, s3_key):
-        self.logger.info("Deleting {} from stage".format(s3_key))
-        self.uploadClient.delete_object(stream, s3_key)
+        """Delete file from snowflake stage"""
+        self.logger.info('Deleting %s from stage', format(s3_key))
+        self.upload_client.delete_object(stream, s3_key)
 
     def get_stage_name(self, stream):
+        """Generate snowflake stage name"""
         stage = self.connection_config.get('stage', None)
         if stage:
             return stage
@@ -495,15 +406,17 @@ class DbSync:
         table_name = self.table_name(stream, False, without_schema=True)
         return f"{self.schema_name}.%{table_name}"
 
-    def load_csv(self, s3_key, count, size_bytes):
+    def load_file(self, s3_key, count, size_bytes):
+        """Load a supported file type from snowflake stage into target table"""
         stream_schema_message = self.stream_schema_message
         stream = stream_schema_message['stream']
-        self.logger.info("Loading {} rows into '{}'".format(count, self.table_name(stream, False)))
+        self.logger.info("Loading %d rows into '%s'", count, self.table_name(stream, False))
 
         # Get list if columns with types
         columns_with_trans = [
             {
                 "name": safe_column_name(name),
+                "json_element_name": json_element_name(name),
                 "trans": column_trans(schema)
             }
             for (name, schema) in self.flatten_schema.items()
@@ -516,30 +429,15 @@ class DbSync:
 
                 # Insert or Update with MERGE command if primary key defined
                 if len(self.stream_schema_message['key_properties']) > 0:
-                    merge_sql = """MERGE INTO {} t
-                        USING (
-                            SELECT {}
-                              FROM '@{}/{}'
-                              (FILE_FORMAT => '{}')) s
-                        ON {}
-                        WHEN MATCHED THEN
-                            UPDATE SET {}
-                        WHEN NOT MATCHED THEN
-                            INSERT ({})
-                            VALUES ({})
-                    """.format(
-                        self.table_name(stream, False),
-                        ', '.join(["{}(${}) {}".format(c['trans'], i + 1, c['name']) for i, c in
-                                   enumerate(columns_with_trans)]),
-                        self.get_stage_name(stream),
-                        s3_key,
-                        self.connection_config['file_format'],
-                        self.primary_key_merge_condition(),
-                        ', '.join(['{}=s.{}'.format(c['name'], c['name']) for c in columns_with_trans]),
-                        ', '.join([c['name'] for c in columns_with_trans]),
-                        ', '.join(['s.{}'.format(c['name']) for c in columns_with_trans])
-                    )
-                    self.logger.debug("Running query: {}".format(merge_sql))
+                    merge_sql = self.file_format.formatter.create_merge_sql(table_name=self.table_name(stream, False),
+                                                                            stage_name=self.get_stage_name(stream),
+                                                                            s3_key=s3_key,
+                                                                            file_format_name=
+                                                                                self.connection_config['file_format'],
+                                                                            columns=columns_with_trans,
+                                                                            pk_merge_condition=
+                                                                                self.primary_key_merge_condition())
+                    self.logger.debug('Running query: %s', merge_sql)
                     cur.execute(merge_sql)
 
                     # Get number of inserted and updated records - MERGE does insert and update
@@ -550,16 +448,13 @@ class DbSync:
 
                 # Insert only with COPY command if no primary key
                 else:
-                    copy_sql = """COPY INTO {} ({}) FROM '@{}/{}'
-                        FILE_FORMAT = (format_name='{}')
-                    """.format(
-                        self.table_name(stream, False),
-                        ', '.join([c['name'] for c in columns_with_trans]),
-                        self.get_stage_name(stream),
-                        s3_key,
-                        self.connection_config['file_format'],
-                    )
-                    self.logger.debug("Running query: {}".format(copy_sql))
+                    copy_sql = self.file_format.formatter.create_copy_sql(table_name=self.table_name(stream, False),
+                                                                          stage_name=self.get_stage_name(stream),
+                                                                          s3_key=s3_key,
+                                                                          file_format_name=
+                                                                            self.connection_config['file_format'],
+                                                                          columns=columns_with_trans)
+                    self.logger.debug('Running query: %s', copy_sql)
                     cur.execute(copy_sql)
 
                     # Get number of inserted records - COPY does insert only
@@ -567,19 +462,22 @@ class DbSync:
                     if len(results) > 0:
                         inserts = results[0].get('rows_loaded', 0)
 
-                self.logger.info('Loading into {}: {}'.format(
+                self.logger.info('Loading into %s: %s',
                     self.table_name(stream, False),
-                    json.dumps({'inserts': inserts, 'updates': updates, 'size_bytes': size_bytes})))
+                    json.dumps({'inserts': inserts, 'updates': updates, 'size_bytes': size_bytes}))
 
     def primary_key_merge_condition(self):
+        """Generate SQL join condition on primary keys for merge SQL statements"""
         stream_schema_message = self.stream_schema_message
         names = primary_column_names(stream_schema_message)
-        return ' AND '.join(['s.{} = t.{}'.format(c, c) for c in names])
+        return ' AND '.join(['s.{0} = t.{0}'.format(c) for c in names])
 
     def column_names(self):
+        """Get list of columns in the schema"""
         return [safe_column_name(name) for name in self.flatten_schema]
 
     def create_table_query(self, is_temporary=False):
+        """Generate CREATE TABLE SQL"""
         stream_schema_message = self.stream_schema_message
         columns = [
             column_clause(
@@ -590,7 +488,7 @@ class DbSync:
         ]
 
         primary_key = ["PRIMARY KEY ({})".format(', '.join(primary_column_names(stream_schema_message)))] \
-            if len(stream_schema_message['key_properties']) else []
+            if len(stream_schema_message.get('key_properties', [])) > 0 else []
 
         return 'CREATE {}TABLE IF NOT EXISTS {} ({}) {}'.format(
             'TEMP ' if is_temporary else '',
@@ -600,18 +498,22 @@ class DbSync:
         )
 
     def grant_usage_on_schema(self, schema_name, grantee):
+        """Grant usage on schema"""
         query = "GRANT USAGE ON SCHEMA {} TO ROLE {}".format(schema_name, grantee)
-        self.logger.info("Granting USAGE privilege on '{}' schema to '{}'... {}".format(schema_name, grantee, query))
+        self.logger.info("Granting USAGE privilege on '%s' schema to '%s'... %s", schema_name, grantee, query)
         self.query(query)
 
+    # pylint: disable=invalid-name
     def grant_select_on_all_tables_in_schema(self, schema_name, grantee):
+        """Grant select on all tables in schema"""
         query = "GRANT SELECT ON ALL TABLES IN SCHEMA {} TO ROLE {}".format(schema_name, grantee)
         self.logger.info(
-            "Granting SELECT ON ALL TABLES privilege on '{}' schema to '{}'... {}".format(schema_name, grantee, query))
+            "Granting SELECT ON ALL TABLES privilege on '%s' schema to '%s'... %s", schema_name, grantee, query)
         self.query(query)
 
     @classmethod
-    def grant_privilege(self, schema, grantees, grant_method):
+    def grant_privilege(cls, schema, grantees, grant_method):
+        """Grant privileges on target schema"""
         if isinstance(grantees, list):
             for grantee in grantees:
                 grant_method(schema, grantee)
@@ -619,12 +521,14 @@ class DbSync:
             grant_method(schema, grantees)
 
     def delete_rows(self, stream):
+        """Hard delete rows from target table"""
         table = self.table_name(stream, False)
         query = "DELETE FROM {} WHERE _sdc_deleted_at IS NOT NULL".format(table)
-        self.logger.info("Deleting rows from '{}' table... {}".format(table, query))
-        self.logger.info("DELETE {}".format(len(self.query(query))))
+        self.logger.info("Deleting rows from '%s' table... %s", table, query)
+        self.logger.info('DELETE %d', len(self.query(query)))
 
     def create_schema_if_not_exists(self):
+        """Create target schema if not exists"""
         schema_name = self.schema_name
         schema_rows = 0
 
@@ -637,7 +541,7 @@ class DbSync:
 
         if len(schema_rows) == 0:
             query = "CREATE SCHEMA IF NOT EXISTS {}".format(schema_name)
-            self.logger.info("Schema '{}' does not exist. Creating... {}".format(schema_name, query))
+            self.logger.info("Schema '%s' does not exist. Creating... %s", schema_name, query)
             self.query(query)
 
             self.grant_privilege(schema_name, self.grantees, self.grant_usage_on_schema)
@@ -646,7 +550,8 @@ class DbSync:
             if self.table_cache:
                 self.table_cache = self.get_table_columns(table_schemas=[self.schema_name])
 
-    def get_tables(self, table_schemas=[]):
+    def get_tables(self, table_schemas=None):
+        """Get list of tables of certain schema(s) from snowflake metadata"""
         tables = []
         if table_schemas:
             for schema in table_schemas:
@@ -671,16 +576,15 @@ class DbSync:
                 # Regexp to extract snowflake error code and message from the exception message
                 # Do nothing if schema not exists
                 except snowflake.connector.errors.ProgrammingError as exc:
-                    if re.match('002043 \(02000\):.*\n.*does not exist.*', str(sys.exc_info()[1])):
-                        pass
-                    else:
+                    if not re.match(r'002043 \(02000\):.*\n.*does not exist.*', str(sys.exc_info()[1])):
                         raise exc
         else:
             raise Exception("Cannot get table columns. List of table schemas empty")
 
         return tables
 
-    def get_table_columns(self, table_schemas=[]):
+    def get_table_columns(self, table_schemas=None):
+        """Get list of columns and tables of certain schema(s) from snowflake metadata"""
         table_columns = []
         if table_schemas:
             for schema in table_schemas:
@@ -725,9 +629,8 @@ class DbSync:
                 # Regexp to extract snowflake error code and message from the exception message
                 # Do nothing if schema not exists
                 except snowflake.connector.errors.ProgrammingError as exc:
-                    if re.match('002003 \(02000\):.*\n.*does not exist or not authorized.*', str(sys.exc_info()[1])):
-                        pass
-                    else:
+                    if not re.match(r'002003 \(02000\):.*\n.*does not exist or not authorized.*',
+                                    str(sys.exc_info()[1])):
                         raise exc
 
         else:
@@ -736,9 +639,11 @@ class DbSync:
         return table_columns
 
     def refresh_table_cache(self):
+        """Refreshes the internal table cache"""
         self.table_cache = self.get_table_columns([self.schema_name])
 
     def update_columns(self):
+        """Adds required but not existing columns the target table according to the schema"""
         stream_schema_message = self.stream_schema_message
         stream = stream_schema_message['stream']
         table_name = self.table_name(stream, False, True)
@@ -783,10 +688,6 @@ class DbSync:
                # a TIMESTAMP_TZ column is already available in the target table (i.e. created by fastsync initial load)
                # We need to exclude this conversion otherwise we loose the data that is already populated
                # in the column
-               #
-               # TODO: Support both TIMESTAMP_TZ and TIMESTAMP_NTZ in target-snowflake
-               # when extracting data-time values from JSON
-               # (Check the column_type function for further details)
                column_type(properties_schema).upper() != 'TIMESTAMP_NTZ'
         ]
 
@@ -796,28 +697,32 @@ class DbSync:
             self.add_column(column, stream)
 
         # Refresh table cache if required
-        if self.table_cache and (len(columns_to_add) > 0 or len(columns_to_replace)):
+        if self.table_cache and (columns_to_add or columns_to_replace):
             self.table_cache = self.get_table_columns(table_schemas=[self.schema_name])
 
     def drop_column(self, column_name, stream):
+        """Drops column from an existing table"""
         drop_column = "ALTER TABLE {} DROP COLUMN {}".format(self.table_name(stream, False), column_name)
-        self.logger.info('Dropping column: {}'.format(drop_column))
+        self.logger.info('Dropping column: %s', drop_column)
         self.query(drop_column)
 
     def version_column(self, column_name, stream):
+        """Versions a column in an existing table"""
         version_column = "ALTER TABLE {} RENAME COLUMN {} TO \"{}_{}\"".format(self.table_name(stream, False),
                                                                                column_name,
                                                                                column_name.replace("\"", ""),
                                                                                time.strftime("%Y%m%d_%H%M"))
-        self.logger.info('Dropping column: {}'.format(version_column))
+        self.logger.info('Versioning column: %s', version_column)
         self.query(version_column)
 
     def add_column(self, column, stream):
+        """Adds a new column to an existing table"""
         add_column = "ALTER TABLE {} ADD COLUMN {}".format(self.table_name(stream, False), column)
-        self.logger.info('Adding column: {}'.format(add_column))
+        self.logger.info('Adding column: %s', add_column)
         self.query(add_column)
 
     def sync_table(self):
+        """Creates or alters the target table according to the schema"""
         stream_schema_message = self.stream_schema_message
         stream = stream_schema_message['stream']
         table_name = self.table_name(stream, False, True)
@@ -833,7 +738,7 @@ class DbSync:
 
         if len(found_tables) == 0:
             query = self.create_table_query()
-            self.logger.info("Table '{}' does not exist. Creating...".format(table_name_with_schema))
+            self.logger.info('Table %s does not exist. Creating...', table_name_with_schema)
             self.query(query)
 
             self.grant_privilege(self.schema_name, self.grantees, self.grant_select_on_all_tables_in_schema)
@@ -842,5 +747,5 @@ class DbSync:
             if self.table_cache:
                 self.table_cache = self.get_table_columns(table_schemas=[self.schema_name])
         else:
-            self.logger.info("Table '{}' exists".format(table_name_with_schema))
+            self.logger.info('Table %s exists', table_name_with_schema)
             self.update_columns()
